"""
title: Animated Actor Level Of Detail Example
by: fabius astelix @ 2009-04
references: http://www.panda3d.org/phpbb2/viewtopic.php?p=36317#36317
revisers:
summary: simple demo to show how to set up a LOD animated actor
\note @ today, if inside the actor character animation the mesh moves changing his distance from the lens, actually the lod system is not aware of that therefore there is no LOD change in that case - it is detected just for node movements.

\todo aggiungere fondale
"""
# Standard imports
import math, sys, time
# Dojo stuff
if not '_P3DPOD_' in globals():
  from pandac.PandaModules import loadPrcFileData
  loadPrcFileData("", '''#
model-path $MAIN_DIR/data/models/
model-path $MAIN_DIR/data/fonts/
model-path $MAIN_DIR/data/shaders/
sound-path $MAIN_DIR/data/sounds/
texture-path $MAIN_DIR/data/textures/
particle-path $MAIN_DIR/data/particles/
  ''')
# Panda imports
from direct.gui.OnscreenText import OnscreenText
from direct.showbase import DirectObject
from direct.showbase.DirectObject import DirectObject
from direct.actor import Actor
import direct.directbase.DirectStart
from direct.task import Task
from pandac.PandaModules import *
#=====================================================================
#
def CreateTextLabel(
    text,
    color,
    i,
    xStart=-1.3,
    yStart=0.95,
    yOffset = 0.1,
    tFont=None
  ):
  if tFont==None:
    return OnscreenText(
      text = text,
      pos = (xStart, yStart-yOffset*i),
      fg=color,
      mayChange = True,
      align=TextNode.ALeft
    )
  else:
    return OnscreenText(
      text = text,
      pos = (xStart, yStart-yOffset*i),
      fg=color,
      mayChange = True,
      align=TextNode.ALeft,
      font=tFont
    )
#=====================================================================
#
class World(DirectObject):
  #======================================================================
  #
  def __init__(self):

    # *** Setup world dimensions
    maxWorldDim = 1000
    worldDims = [maxWorldDim,maxWorldDim,maxWorldDim]
    base.camLens.setNearFar(0.1,2.5*maxWorldDim)

    # the frame rate meter
    base.setFrameRateMeter(True)

    # *** Setup text displays
    textXStart=-1.3
    textYStart=0.95
    textYOffset=0.1
    self.mTitleDisplay = \
    CreateTextLabel(
      "LOD Node Test   (w=wireframe toggle)",
      (1.0, 1.0, -1.0, 1.0),
      1,
      textXStart,
      textYStart,
      textYOffset
    )
    self.mMessageDisplay = \
    CreateTextLabel(
      '> ',
      (0.0,1.0,0.0,1.0),
      3,
      textXStart,
      textYStart,
      textYOffset
    )

    # *** Setup lighting
    lightLevel=0.7
    lightPos=(0.0,-10.0,10.0)
    lightHpr=(0.0,-26.0,0.0)
    dlight = DirectionalLight('dlight')
    dlight.setColor(VBase4(lightLevel, lightLevel, lightLevel, 1))
    dlnp = render.attachNewNode(dlight.upcastToPandaNode())
    dlnp.setHpr(lightHpr[0],lightHpr[1],lightHpr[2])
    dlnp.setPos(lightPos[0],lightPos[1],lightPos[2])
    render.setLight(dlnp)

    alight = AmbientLight('alight')
    alight.setColor(VBase4(0.2, 0.2, 0.2, 1))
    alnp = render.attachNewNode(alight.upcastToPandaNode())

    # *** Setup scene
    base.setBackgroundColor(0.0,0.1,0.7,1.0)

    # *** Setup input events
    self.setupInput()

    self._showtext("Loading...")

    #remember that the actor instance must not be volatile
    self.subject = Actor.Actor(
      {
        'lod0': "slim-panda-lod0.egg.pz",
        'lod1': "slim-panda-lod1.egg.pz",
        'lod2': "slim-panda-lod2.egg.pz",
        'lod3': "slim-panda-lod3.egg.pz",
      },
      {
        'jump': 'slim-anim-jump00.egg.pz',
      },
    )
    self.subject.setLOD('lod0', 5, 0)
    self.subject.setLOD('lod1', 10, 5)
    self.subject.setLOD('lod2', 40, 10)
    self.subject.setLOD('lod3', 999999, 40)
    self.subject.reparentTo(base.render)
    self.subject.setPos(0, 0, 0)
    self.subject.loop('jump')

    # Wireframe on
    base.toggleWireframe()

    # *** Done, setup start camera management task
    taskMgr.add(self.cameraTask,'tsk_cam')
  #======================================================================
  #
  def _showtext(self, t):
    self.mMessageDisplay.setText(t)
    for i in range(4):
      base.graphicsEngine.renderFrame()
      time.sleep(0.3)
  #======================================================================
  #
  def cameraTask(self, task):
    DELTA=.002
    MINDIST=1
    MAXDIST=60.
    speed=18.0

    if not hasattr(self, 'cam_distance'):
      self.cam_distance=MAXDIST
      self.delta=-DELTA
    else:
      self.cam_distance+=self.delta*task.time
      if self.cam_distance < MINDIST:
        self.delta=DELTA
        self.cam_distance=MINDIST
      elif self.cam_distance > MAXDIST:
        self.delta=-DELTA
        self.cam_distance=MAXDIST

    angledegrees = task.time * speed
    angleradians = angledegrees * (math.pi / 180.0)
    distanceradius = self.cam_distance
    base.camera.setPos(
      distanceradius*math.sin(angleradians),
      -distanceradius*math.cos(angleradians),6
    )
    base.camera.setHpr(angledegrees, 0, 0)
    base.camera.lookAt(self.subject)
    #
    pos=base.camera.getPos()
    s = "Camera Pos : %12.4f %12.4f %12.4f (d:%12.4f)" % (pos[0], pos[1], pos[2], self.cam_distance)
    self.mMessageDisplay.setText(s)
    return Task.cont
  #======================================================================
  #
  def setupInput(self) :
    base.disableMouse()
    base.accept('w', base.toggleWireframe)
  #------------------------------------------------------------
  #
  def onToggleWire(self, event=None): base.toggleWireframe()

#-------------------------------------------------------
pea=World()
if __name__ == "__main__":
  if not '_P3DPOD_' in globals(): run()
